unit mainform;

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, SysUtils, 
  Registry, ExtCtrls, StdCtrls, ComCtrls, Dialogs;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Button2: TButton;
    Timer1: TTimer;
    CheckBox1: TCheckBox;
    Shape1: TShape;
    Label2: TLabel;
    Button1: TButton;
    ProgressBar1: TProgressBar;
    Image1: TImage;
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);

  private
    { Private declarations }

  public
    { Public declarations }

  end;

const
   cERROR_BUFFER_TOO_SMALL = 603;
   cRAS_MaxEntryName = 256;
   cRAS_MaxDeviceName = 128;
   cRAS_MaxDeviceType = 16;

type
   TConnectionType = (ctNone, ctProxy, ctDialup);
   ERasError = class(Exception);
   HRASConn = DWord;
   PRASConn = ^TRASConn;
   TRASConn = record dwSize: DWORD;
   rasConn: HRASConn;
   szEntryName: Array[0..cRAS_MaxEntryName] Of Char;
   szDeviceType : Array[0..cRAS_MaxDeviceType] Of Char;
   szDeviceName : Array [0..cRAS_MaxDeviceName] of char; end;
   TRasEnumConnections = function (RASConn: PrasConn; var BufSize: DWord;

var
   Connections: DWord):
   LongInt; stdcall;

var
   Form1: TForm1;
   HasBeenConnected : Boolean;
   ShutdownCounter : integer;

function ConnectedToInternet : TConnectionType;
function RasConnectionCount : Integer;
function ShutdownWindows(Flag: word) : Boolean;

implementation

function ConnectedToInternet: TConnectionType;
var Reg : TRegistry; bUseProxy : Boolean; UseProxy : LongWord;
begin
 Result := ctNone;
 Reg := TRegistry.Create;
 with REG do try
  try RootKey := HKEY_CURRENT_USER;
  if OpenKey('Software\Microsoft\WindowsCurrentVersion\Internet settings',False)
  then begin
   if GetDataType('ProxyEnable') = rdBinary then
   ReadBinaryData('ProxyEnable', UseProxy, SizeOf(LongWord) )
   else begin
    bUseProxy := ReadBool('ProxyEnable');
    if bUseProxy then UseProxy := 1 else UseProxy := 0;
    end;
   end;
  except end;
 finally Free; 
end; 
 if Result = ctNone 
 then begin 
  if RasConnectionCount > 0 then Result := ctDialup;
 end;
end;

function RasConnectionCount : Integer;
var RasDLL : HInst;
Conns : Array[1..4] of TRasConn;
RasEnums : TRasEnumConnections;
BufSize, NumConns : DWord;
RasResult : Longint;
begin
 Result := 0;
 RasDLL := LoadLibrary('rasapi32.dll');
 if RasDLL = 0 then exit;
 try RasEnums := GetProcAddress(RasDLL,'RasEnumConnectionsA');
 if @RasEnums = nil then
  raise ERasError.Create('RasEnumConnectionsA not found in rasapi32.dll');
  Conns[1].dwSize := Sizeof (Conns[1]);
  BufSize := SizeOf(Conns);
  RasResult := RasEnums(@Conns, BufSize, NumConns);
 If (RasResult = 0) or (Result = cERROR_BUFFER_TOO_SMALL) then
  Result := NumConns;
 finally FreeLibrary(RasDLL);
 end;
end;

function ShutDownWindows(Flag: word): Boolean;
 var
  TokenPriv: TTokenPrivileges;
  H: DWord;
  HToken: THandle;
begin
 if Win32Platform = VER_PLATFORM_WIN32_NT then
  begin
   OpenProcessToken(GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES,HToken);
   LookUpPrivilegeValue(NIL, 'SeShutdownPrivilege', TokenPriv.Privileges[0].Luid);
   TokenPriv.PrivilegeCount := 1;
   TokenPriv.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
   H := 0;
   AdjustTokenPrivileges(HToken, FALSE, TokenPriv, 0, PTokenPrivileges(NIL)^, H);
   CloseHandle(HToken);
  end;
 Result := ExitWindowsEx(Flag, 0);
end;


{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
 Image1.Picture.Graphic := Application.Icon;
 HasBeenConnected := false;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var Status : TConnectionType; 
sMsg : string; 
AllowShutdown : Boolean;
begin
 Status := ConnectedToInternet;
 AllowShutdown := CheckBox1.Checked;
 if Status = ctDialup then
 begin
  with Shape1.Brush do begin
   if Color = clLime then
   Color := clBtnFace else
   Color := clLime;
  end;
  sMsg := 'Online';
  HasBeenConnected := true;
 end;
 if Status = ctNone then
 begin
  Shape1.Brush.Color := clRed;
  sMsg := 'Offline';
  if HasBeenConnected = true
  then begin
   if AllowShutdown = true
   then try begin
    Label2.Enabled := true;
    ProgressBar1.Enabled := true;
    Button1.Enabled := true;
    CheckBox1.Enabled := false;
    ProgressBar1.Position := ProgressBar1.Position + 1;
    Label2.Caption := 'Shutdown in ' + (IntToStr(30-(ProgressBar1.Position))) + ' seconds.';
    if ProgressBar1.Position >= 30 then ShutdownWindows(EWX_POWEROFF);
   end except begin
    MessageDlg('Unable to shutdown Windows.', mtError, [mbOK], 0);
    ProgressBar1.Position := 0;
    CheckBox1.Enabled := true;
    Button1.Enabled := False;
    ProgressBar1.Enabled := false;
    Label2.Caption := 'Shutdown Timer';
    Label2.Enabled := false;
   end;
   end;
  end;
 end;
 Label1.Caption := sMsg;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
var Status : TConnectionType;
begin
 Status := ConnectedToInternet;
 if Status = ctNone then HasBeenConnected := false;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
 ShutdownWindows(EWX_POWEROFF);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
 Close;
end;

end.

